@@ -14,6 +14,7 @@
 #include <htslib/sam.h>
 
 #include "util.h"
+#include "assemble.h"
 
 namespace torali
 {
@@ -64,7 +65,7 @@ namespace torali
     if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP | BAM_FSECONDARY | BAM_FSUPPLEMENTARY)) readStart = -1;
     typedef typename TReadBp::mapped_type TJunctionVector;
     typename TReadBp::iterator it = readBp.find(seed);
-    int32_t seqlen = sequenceLength(rec);
+    int32_t seqlen = readLength(rec);
     if (sp <= seqlen) {
       if (rec->core.flag & BAM_FREVERSE) {
 	if (it != readBp.end()) it->second.push_back(Junction(fw, scleft, rec->core.tid, readStart, rp, seqlen - sp, rec->core.qual));
@@ -274,206 +275,138 @@ namespace torali
   }
 
 
-  template<typename TConfig, typename TReadBp>
+  template<typename TConfig, typename TValidRegion, typename TReadBp>
   inline void
-  findJunctions(TConfig const& c, TReadBp& readBp, std::vector<StructuralVariantRecord>& svs) {
-    samFile* samfile = sam_open(c.files[0].string().c_str(), "r");
-    hts_set_fai_filename(samfile, c.genome.string().c_str());
-    hts_idx_t* idx = sam_index_load(samfile, c.files[0].string().c_str());
-    bam_hdr_t* hdr = sam_hdr_read(samfile);
+  findJunctions(TConfig const& c, TValidRegion const& validRegions, TReadBp& readBp) {
+    typedef typename TValidRegion::value_type TChrIntervals;
+
+    // Open file handles
+    typedef std::vector<samFile*> TSamFile;
+    typedef std::vector<hts_idx_t*> TIndex;
+    TSamFile samfile(c.files.size());
+    TIndex idx(c.files.size());
+    for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+      samfile[file_c] = sam_open(c.files[file_c].string().c_str(), "r");
+      hts_set_fai_filename(samfile[file_c], c.genome.string().c_str());
+      idx[file_c] = sam_index_load(samfile[file_c], c.files[file_c].string().c_str());
+    }
+    bam_hdr_t* hdr = sam_hdr_read(samfile[0]);
     
     // Parse genome chr-by-chr
     boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
     std::cout << '[' << boost::posix_time::to_simple_string(now) << "] " << "Split-read scanning" << std::endl;
     boost::progress_display show_progress( hdr->n_targets );
 
-    // Track reference reads
-    bool trackRef = false;
-    if (!svs.empty()) trackRef = true;
-    typedef boost::dynamic_bitset<> TBitSet;
-    TBitSet bpOccupied;
-    char* seq = NULL;
-    faidx_t* fai = fai_load(c.genome.string().c_str());
-    
     // Iterate chromosomes
     for(int32_t refIndex=0; refIndex < (int32_t) hdr->n_targets; ++refIndex) {
       ++show_progress;
+      if (validRegions[refIndex].empty()) continue;
 
-      // Flag breakpoints
-      if (trackRef) {
-	bpOccupied.resize(hdr->target_len[refIndex]);
-	for(uint32_t i = 0; i < hdr->target_len[refIndex]; ++i) bpOccupied[i] = 0;
-	for(uint32_t i = 0; i < svs.size(); ++i) {
-	  if (svs[i].chr == refIndex) bpOccupied[svs[i].svStart] = 1;
-	  if (svs[i].chr2 == refIndex) bpOccupied[svs[i].svEnd] = 1;
-	}
-	std::string tname(hdr->target_name[refIndex]);
-	int32_t seqlen = -1;
-	seq = faidx_fetch_seq(fai, tname.c_str(), 0, hdr->target_len[refIndex], &seqlen);
-      }	else {
-	seq = NULL;
-      }
+      // Collect reads from all samples
+      for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+	// Read alignments
+	for(typename TChrIntervals::const_iterator vRIt = validRegions[refIndex].begin(); vRIt != validRegions[refIndex].end(); ++vRIt) {
+	  hts_itr_t* iter = sam_itr_queryi(idx[file_c], refIndex, vRIt->lower(), vRIt->upper());
+	  bam1_t* rec = bam_init1();
+	  while (sam_itr_next(samfile[file_c], iter, rec) >= 0) {
 
-      // Parse BAM
-      hts_itr_t* iter = sam_itr_queryi(idx, refIndex, 0, c.chrlen[refIndex]);
-      bam1_t* rec = bam_init1();
-      while (sam_itr_next(samfile, iter, rec) >= 0) {
-	// Keep secondary alignments
-	if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP)) continue;
-	if ((rec->core.qual < c.minMapQual) || (rec->core.tid<0)) continue;
+	    // Keep secondary alignments
+	    if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP)) continue;
+	    if ((rec->core.qual < c.minMapQual) || (rec->core.tid<0)) continue;
 
-	std::size_t seed = hash_lr(rec);
-	uint32_t rp = rec->core.pos; // reference pointer
-	uint32_t sp = 0; // sequence pointer
-      
-	// Parse the CIGAR
-	uint32_t* cigar = bam_get_cigar(rec);
-	for (std::size_t i = 0; i < rec->core.n_cigar; ++i) {
-	  if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF)) {
-	    // Fetch reference alignments
-	    if (trackRef) {
-	      int32_t bpHit = 0;
-	      for(uint32_t k = 0; k < bam_cigar_oplen(cigar[i]); ++k) {
-		if (bpOccupied[rp]) bpHit = rp;
-		++sp;
-		++rp;
-	      }
-	      if (bpHit) {
-		int32_t bg = std::max((int32_t) 0, bpHit - c.minimumFlankSize);
-		int32_t ed = std::min((int32_t) hdr->target_len[refIndex], bpHit + c.minimumFlankSize);
-		if ((bg < rec->core.pos) || ((uint32_t) ed > rec->core.pos + alignmentLength(rec))) continue;
-
-		// Get sequence
-		std::string sequence;
-		sequence.resize(rec->core.l_qseq);
-		uint8_t* seqptr = bam_get_seq(rec);
-		for (int i = 0; i < rec->core.l_qseq; ++i) sequence[i] = "=ACMGRSVTWYHKDBN"[bam_seqi(seqptr, i)];
-		
-		// Re-iterate cigar to track alignment		
-		int32_t rit = rec->core.pos; // reference pointer
-		int32_t sit = 0; // sequence pointer
-		std::string refAlign;
-		std::string altAlign;
-		for (std::size_t ij = 0; ij < rec->core.n_cigar; ++ij) {
-		  if (rit >= ed) break;
-		  if ((bam_cigar_op(cigar[ij]) == BAM_CMATCH) || (bam_cigar_op(cigar[ij]) == BAM_CEQUAL) || (bam_cigar_op(cigar[ij]) == BAM_CDIFF)) {
-		    for(uint32_t k = 0; k < bam_cigar_oplen(cigar[ij]); ++k) {
-		      if ((rit >= bg) && (rit < ed)) {
-			refAlign.push_back(seq[rit]);
-			altAlign.push_back(sequence[sit]);
+	    std::size_t seed = hash_lr(rec);
+	    //std::cerr << bam_get_qname(rec) << '\t' << seed << std::endl;
+	    uint32_t rp = rec->core.pos; // reference pointer
+	    uint32_t sp = 0; // sequence pointer
+	    
+	    // Parse the CIGAR
+	    uint32_t* cigar = bam_get_cigar(rec);
+	    for (std::size_t i = 0; i < rec->core.n_cigar; ++i) {
+	      if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF)) {
+		sp += bam_cigar_oplen(cigar[i]);
+		rp += bam_cigar_oplen(cigar[i]);
+	      } else if (bam_cigar_op(cigar[i]) == BAM_CDEL) {
+		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, false);
+		rp += bam_cigar_oplen(cigar[i]);
+		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) { // Try look-ahead
+		  uint32_t spOrig = sp;
+		  uint32_t rpTmp = rp;
+		  uint32_t spTmp = sp;
+		  uint32_t dlen = bam_cigar_oplen(cigar[i]);
+		  for (std::size_t j = i + 1; j < rec->core.n_cigar; ++j) {
+		    if ((bam_cigar_op(cigar[j]) == BAM_CMATCH) || (bam_cigar_op(cigar[j]) == BAM_CEQUAL) || (bam_cigar_op(cigar[j]) == BAM_CDIFF)) {
+		      spTmp += bam_cigar_oplen(cigar[j]);
+		      rpTmp += bam_cigar_oplen(cigar[j]);
+		      if ((double) (spTmp - sp) / (double) (dlen + (rpTmp - rp)) > c.indelExtension) break;
+		    } else if (bam_cigar_op(cigar[j]) == BAM_CDEL) {
+		      rpTmp += bam_cigar_oplen(cigar[j]);
+		      if (bam_cigar_oplen(cigar[j]) > c.minRefSep) {
+			// Extend deletion
+			dlen += (rpTmp - rp);
+			rp = rpTmp;
+			sp = spTmp;
+			i = j;
 		      }
-		      ++rit;
-		      ++sit;
-		    }
-		  } else if ((bam_cigar_op(cigar[ij]) == BAM_CDEL) || (bam_cigar_op(cigar[ij]) == BAM_CREF_SKIP)) {
-		    for(uint32_t k = 0; k < bam_cigar_oplen(cigar[ij]); ++k) {
-		      if ((rit >= bg) && (rit < ed)) {
-			refAlign.push_back(seq[rit]);
-			altAlign.push_back('-');
-		      }
-		      ++rit;
-		    }
-		  } else if (bam_cigar_op(cigar[ij]) == BAM_CINS) {
-		    for(uint32_t k = 0; k < bam_cigar_oplen(cigar[ij]); ++k) {
-		      if ((rit >= bg) && (rit < ed)) {
-			refAlign.push_back('-');
-			altAlign.push_back(sequence[sit]);
-		      }
-		      ++sit;
-		    }
-		  } else if ((bam_cigar_op(cigar[ij]) == BAM_CSOFT_CLIP) || (bam_cigar_op(cigar[ij]) == BAM_CHARD_CLIP)) {
-		    sit += bam_cigar_oplen(cigar[ij]);
+		    } else if (bam_cigar_op(cigar[j]) == BAM_CINS) {
+		      if (bam_cigar_oplen(cigar[j]) > c.minRefSep) break; // No extension
+		      spTmp += bam_cigar_oplen(cigar[j]);
+		    } else break; // No extension
 		  }
+		  _insertJunction(readBp, seed, rec, rp, spOrig, true);
 		}
-		std::cerr << refAlign << std::endl;
-		std::cerr << altAlign << std::endl;
-		std::cerr << std::endl;
-	      }
-	    } else {
-	      sp += bam_cigar_oplen(cigar[i]);
-	      rp += bam_cigar_oplen(cigar[i]);
-	    }
-	  } else if (bam_cigar_op(cigar[i]) == BAM_CDEL) {
-	    if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, false);
-	    rp += bam_cigar_oplen(cigar[i]);
-	    if (bam_cigar_oplen(cigar[i]) > c.minRefSep) { // Try look-ahead
-	      uint32_t spOrig = sp;
-	      uint32_t rpTmp = rp;
-	      uint32_t spTmp = sp;
-	      uint32_t dlen = bam_cigar_oplen(cigar[i]);
-	      for (std::size_t j = i + 1; j < rec->core.n_cigar; ++j) {
-		if ((bam_cigar_op(cigar[j]) == BAM_CMATCH) || (bam_cigar_op(cigar[j]) == BAM_CEQUAL) || (bam_cigar_op(cigar[j]) == BAM_CDIFF)) {
-		  spTmp += bam_cigar_oplen(cigar[j]);
-		  rpTmp += bam_cigar_oplen(cigar[j]);
-		  if ((double) (spTmp - sp) / (double) (dlen + (rpTmp - rp)) > c.indelExtension) break;
-		} else if (bam_cigar_op(cigar[j]) == BAM_CDEL) {
-		  rpTmp += bam_cigar_oplen(cigar[j]);
-		  if (bam_cigar_oplen(cigar[j]) > c.minRefSep) {
-		    // Extend deletion
-		    dlen += (rpTmp - rp);
-		    rp = rpTmp;
-		    sp = spTmp;
-		    i = j;
+	      } else if (bam_cigar_op(cigar[i]) == BAM_CINS) {
+		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, false);
+		sp += bam_cigar_oplen(cigar[i]);
+		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) { // Try look-ahead
+		  uint32_t rpOrig = rp;
+		  uint32_t rpTmp = rp;
+		  uint32_t spTmp = sp;
+		  uint32_t ilen = bam_cigar_oplen(cigar[i]);
+		  for (std::size_t j = i + 1; j < rec->core.n_cigar; ++j) {
+		    if ((bam_cigar_op(cigar[j]) == BAM_CMATCH) || (bam_cigar_op(cigar[j]) == BAM_CEQUAL) || (bam_cigar_op(cigar[j]) == BAM_CDIFF)) {
+		      spTmp += bam_cigar_oplen(cigar[j]);
+		      rpTmp += bam_cigar_oplen(cigar[j]);
+		      if ((double) (rpTmp - rp) / (double) (ilen + (spTmp - sp)) > c.indelExtension) break;
+		    } else if (bam_cigar_op(cigar[j]) == BAM_CDEL) {
+		      if (bam_cigar_oplen(cigar[j]) > c.minRefSep) break; // No extension
+		      rpTmp += bam_cigar_oplen(cigar[j]);
+		    } else if (bam_cigar_op(cigar[j]) == BAM_CINS) {
+		      spTmp += bam_cigar_oplen(cigar[j]);
+		      if (bam_cigar_oplen(cigar[j]) > c.minRefSep) {
+			// Extend insertion
+			ilen += (spTmp - sp);
+			rp = rpTmp;
+			sp = spTmp;
+			i = j;
+		      }
+		    } else {
+		      break; // No extension
+		    }
 		  }
-		} else if (bam_cigar_op(cigar[j]) == BAM_CINS) {
-		  if (bam_cigar_oplen(cigar[j]) > c.minRefSep) break; // No extension
-		  spTmp += bam_cigar_oplen(cigar[j]);
-		} else break; // No extension
-	      }
-	      _insertJunction(readBp, seed, rec, rp, spOrig, true);
-	    }
-	  } else if (bam_cigar_op(cigar[i]) == BAM_CINS) {
-	    if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, false);
-	    sp += bam_cigar_oplen(cigar[i]);
-	    if (bam_cigar_oplen(cigar[i]) > c.minRefSep) { // Try look-ahead
-	      uint32_t rpOrig = rp;
-	      uint32_t rpTmp = rp;
-	      uint32_t spTmp = sp;
-	      uint32_t ilen = bam_cigar_oplen(cigar[i]);
-	      for (std::size_t j = i + 1; j < rec->core.n_cigar; ++j) {
-		if ((bam_cigar_op(cigar[j]) == BAM_CMATCH) || (bam_cigar_op(cigar[j]) == BAM_CEQUAL) || (bam_cigar_op(cigar[j]) == BAM_CDIFF)) {
-		  spTmp += bam_cigar_oplen(cigar[j]);
-		  rpTmp += bam_cigar_oplen(cigar[j]);
-		  if ((double) (rpTmp - rp) / (double) (ilen + (spTmp - sp)) > c.indelExtension) break;
-		} else if (bam_cigar_op(cigar[j]) == BAM_CDEL) {
-		  if (bam_cigar_oplen(cigar[j]) > c.minRefSep) break; // No extension
-		  rpTmp += bam_cigar_oplen(cigar[j]);
-		} else if (bam_cigar_op(cigar[j]) == BAM_CINS) {
-		  spTmp += bam_cigar_oplen(cigar[j]);
-		  if (bam_cigar_oplen(cigar[j]) > c.minRefSep) {
-		  // Extend insertion
-		    ilen += (spTmp - sp);
-		    rp = rpTmp;
-		    sp = spTmp;
-		    i = j;
-		  }
-		} else {
-		  break; // No extension
+		  _insertJunction(readBp, seed, rec, rpOrig, sp, true);
 		}
+	      } else if (bam_cigar_op(cigar[i]) == BAM_CREF_SKIP) {
+		rp += bam_cigar_oplen(cigar[i]);
+	      } else if ((bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) || (bam_cigar_op(cigar[i]) == BAM_CHARD_CLIP)) {
+		int32_t finalsp = sp;
+		bool scleft = false;
+		if (sp == 0) {
+		  finalsp += bam_cigar_oplen(cigar[i]); // Leading soft-clip / hard-clip
+		  scleft = true;
+		}
+		sp += bam_cigar_oplen(cigar[i]);
+		//std::cerr << bam_get_qname(rec) << ',' << rp << ',' << finalsp << ',' << scleft << std::endl;
+		if (bam_cigar_oplen(cigar[i]) > c.minClip) _insertJunction(readBp, seed, rec, rp, finalsp, scleft);
+	      } else {
+		std::cerr << "Unknown Cigar options" << std::endl;
 	      }
-	      _insertJunction(readBp, seed, rec, rpOrig, sp, true);
 	    }
-	  } else if ((bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) || (bam_cigar_op(cigar[i]) == BAM_CHARD_CLIP)) {
-	    int32_t finalsp = sp;
-	    bool scleft = false;
-	    if (sp == 0) {
-	      finalsp += bam_cigar_oplen(cigar[i]); // Leading soft-clip / hard-clip
-	      scleft = true;
-	    }
-	    sp += bam_cigar_oplen(cigar[i]);
-	    if (bam_cigar_oplen(cigar[i]) > c.minClip) _insertJunction(readBp, seed, rec, rp, finalsp, scleft);
-	  } else if (bam_cigar_op(cigar[i]) == BAM_CREF_SKIP) {
-	    rp += bam_cigar_oplen(cigar[i]);
-	  } else {
-	    std::cerr << "Unknown Cigar options" << std::endl;
 	  }
+	  bam_destroy1(rec);
+	  hts_itr_destroy(iter);
 	}
       }
-      if (seq != NULL) free(seq);
-      bam_destroy1(rec);
-      hts_itr_destroy(iter);
     }
-    fai_destroy(fai);
 
     // Sort junctions
     for(typename TReadBp::iterator it = readBp.begin(); it != readBp.end(); ++it) {
@@ -482,19 +415,71 @@ namespace torali
 
     // Clean-up
     bam_hdr_destroy(hdr);
-    hts_idx_destroy(idx);
-    sam_close(samfile);
+    for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+      hts_idx_destroy(idx[file_c]);
+      sam_close(samfile[file_c]);
+    }
   }
 
+
   template<typename TConfig, typename TReadBp>
   inline void
   fetchSVs(TConfig const& c, TReadBp& readBp, std::vector<std::vector<SRBamRecord> >& br) {
     // Extract BAM records
-    if ((c.svtype == "ALL") || (c.svtype == "DEL")) selectDeletions(c, readBp, br);
-    if ((c.svtype == "ALL") || (c.svtype == "DUP")) selectDuplications(c, readBp, br);
-    if ((c.svtype == "ALL") || (c.svtype == "INV")) selectInversions(c, readBp, br);
-    if ((c.svtype == "ALL") || (c.svtype == "INS")) selectInsertions(c, readBp, br);
-    if ((c.svtype == "ALL") || (c.svtype == "BND")) selectTranslocations(c, readBp, br);
+    if ((!c.svtcmd) || (c.svtset.find(2) != c.svtset.end())) selectDeletions(c, readBp, br);
+    //if ((!c.svtcmd) || (c.svtset.find(3) != c.svtset.end())) selectDuplications(c, readBp, br);
+    //if ((!c.svtcmd) || (c.svtset.find(0) != c.svtset.end()) || (c.svtset.find(1) != c.svtset.end())) selectInversions(c, readBp, br);
+    if ((!c.svtcmd) || (c.svtset.find(4) != c.svtset.end())) selectInsertions(c, readBp, br);
+    //if ((!c.svtcmd) || (c.svtset.find(5) != c.svtset.end()) || (c.svtset.find(6) != c.svtset.end()) || (c.svtset.find(7) != c.svtset.end()) || (c.svtset.find(8) != c.svtset.end())) selectTranslocations(c, readBp, br);
+  }
+
+  template<typename TConfig, typename TValidRegions, typename TSvtSRBamRecord>
+  inline void
+    _findSRBreakpoints(TConfig const& c, TValidRegions const& validRegions, TSvtSRBamRecord& srBR) {
+    // Breakpoints
+    typedef std::vector<Junction> TJunctionVector;
+    typedef std::map<std::size_t, TJunctionVector> TReadBp;
+    TReadBp readBp;
+    findJunctions(c, validRegions, readBp);
+    fetchSVs(c, readBp, srBR);
+  }
+
+
+  template<typename TConfig, typename TValidRegions, typename TSVs, typename TSRStore>
+  inline void
+  _clusterSRReads(TConfig const& c, TValidRegions const& validRegions, TSVs& svc, TSRStore& srStore) {
+    typedef typename TSRStore::mapped_type TSvPosVector;
+    // Split-reads
+    typedef std::vector<SRBamRecord> TSRBamRecord;
+    typedef std::vector<TSRBamRecord> TSvtSRBamRecord;
+    TSvtSRBamRecord srBR(2 * DELLY_SVT_TRANS, TSRBamRecord());
+    _findSRBreakpoints(c, validRegions, srBR);
+	 	 
+    // Debug
+    //outputSRBamRecords(c, srBR);
+    
+    // Cluster BAM records
+    for(uint32_t svt = 0; svt < srBR.size(); ++svt) {
+      if (srBR[svt].empty()) continue;
+      
+      // Sort
+      std::sort(srBR[svt].begin(), srBR[svt].end(), SortSRBamRecord<SRBamRecord>());
+      
+      // Cluster
+      cluster(c, srBR[svt], svc, c.maxReadSep, svt);
+
+      // Debug
+      //outputStructuralVariants(c, svc, srBR, svt);
+      
+      // Track split-reads
+      for(uint32_t i = 0; i < srBR[svt].size(); ++i) {
+	// Read assigned?
+	if (srBR[svt][i].svid != -1) {
+	  if (srStore.find(srBR[svt][i].id) == srStore.end()) srStore.insert(std::make_pair(srBR[svt][i].id, TSvPosVector()));
+	  srStore[srBR[svt][i].id].push_back(SeqSlice(srBR[svt][i].svid, srBR[svt][i].sstart, srBR[svt][i].inslen, srBR[svt][i].qual));
+	}
+      }
+    }
   }
 
 
@@ -519,21 +504,88 @@ namespace torali
     sam_close(samfile);
   }
 
+  template<typename TConfig, typename TSvtSRBamRecord>
+  inline void
+  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs, TSvtSRBamRecord const& srBR, int32_t const svt) {
+    // Header
+    std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tpeSupport\tsrSupport" << std::endl;
+    
+    // Hash reads
+    typedef std::map<std::size_t, std::string> THashMap;
+    THashMap hm;
+    typedef std::vector<samFile*> TSamFile;
+    typedef std::vector<hts_idx_t*> TIndex;
+    TSamFile samfile(c.files.size());
+    TIndex idx(c.files.size());
+    for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+      samfile[file_c] = sam_open(c.files[file_c].string().c_str(), "r");
+      hts_set_fai_filename(samfile[file_c], c.genome.string().c_str());
+      idx[file_c] = sam_index_load(samfile[file_c], c.files[file_c].string().c_str());
+    }
+    bam_hdr_t* hdr = sam_hdr_read(samfile[0]);
+    for(int32_t refIndex = 0; refIndex < hdr->n_targets; ++refIndex) {
+      for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+	hts_itr_t* iter = sam_itr_queryi(idx[file_c], refIndex, 0, hdr->target_len[refIndex]);
+	bam1_t* rec = bam_init1();
+	while (sam_itr_next(samfile[file_c], iter, rec) >= 0) {
+	  if (rec->core.flag & (BAM_FQCFAIL | BAM_FDUP | BAM_FUNMAP | BAM_FSECONDARY | BAM_FSUPPLEMENTARY)) continue;
+	  std::size_t seed = hash_lr(rec);
+	  std::string qname = bam_get_qname(rec);
+	  if (hm.find(seed) == hm.end()) hm.insert(std::make_pair(seed, qname));
+	  else {
+	    if (hm[seed] != qname) {
+	      std::cerr << "Warning: Hash collision! " << seed << ',' << hm[seed] << ',' << qname << std::endl;
+	    }
+	  }
+	}
+	bam_destroy1(rec);
+	hts_itr_destroy(iter);
+      }
+    }
+    
+    // Track split-reads
+    typedef std::vector<std::string> TReadNameVector;
+    typedef std::vector<TReadNameVector> TSVReadNames;
+    TSVReadNames svReadNames(svs.size(), TReadNameVector());
+    for(uint32_t i = 0; i < srBR[svt].size(); ++i) {
+      if (srBR[svt][i].svid != -1) {
+	svReadNames[srBR[svt][i].svid].push_back(hm[srBR[svt][i].id]);
+      }
+    }
+  
+    // SVs
+    for(uint32_t i = 0; i < svs.size(); ++i) {
+      if (svs[i].svt != svt) continue;
+      std::cerr << hdr->target_name[svs[i].chr] << '\t' << svs[i].svStart << '\t' << hdr->target_name[svs[i].chr2] << '\t' << svs[i].svEnd << '\t' << _addID(svs[i].svt) << '\t' << _addOrientation(svs[i].svt) << '\t' << svs[i].peSupport << '\t' << svs[i].srSupport << '\t';
+      for(uint32_t k = 0; k < svReadNames[svs[i].id].size(); ++k) std::cerr << svReadNames[svs[i].id][k] << ',';
+      std::cerr << std::endl;
+    }
+
+    // Clean-up
+    bam_hdr_destroy(hdr);
+    for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
+      hts_idx_destroy(idx[file_c]);
+      sam_close(samfile[file_c]);
+    }
+  }
+
+
   template<typename TConfig>
   inline void
-  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs) {
+  outputStructuralVariants(TConfig const& c, std::vector<StructuralVariantRecord> const& svs, int32_t const svt) {
     samFile* samfile = sam_open(c.files[0].string().c_str(), "r");
     hts_set_fai_filename(samfile, c.genome.string().c_str());
     bam_hdr_t* hdr = sam_hdr_read(samfile);
 
     // Header
     std::cerr << "chr1\tpos1\tchr2\tpos2\tsvtype\tct\tpeSupport\tsrSupport" << std::endl;
-
+    
     // SVs
     for(uint32_t i = 0; i < svs.size(); ++i) {
+      if (svs[i].svt != svt) continue;
       std::cerr << hdr->target_name[svs[i].chr] << '\t' << svs[i].svStart << '\t' << hdr->target_name[svs[i].chr2] << '\t' << svs[i].svEnd << '\t' << _addID(svs[i].svt) << '\t' << _addOrientation(svs[i].svt) << '\t' << svs[i].peSupport << '\t' << svs[i].srSupport << std::endl;
     }
-
+    
     // Clean-up
     bam_hdr_destroy(hdr);
     sam_close(samfile);
