@@ -104,14 +104,6 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
     int32_t* svend = NULL;
     int32_t ninslen = 0;
     int32_t* inslen = NULL;
-    int32_t npe = 0;
-    int32_t* pe = NULL;
-    int32_t nsr = 0;
-    int32_t* sr = NULL;
-    int32_t nmapq = 0;
-    int32_t* mapq = NULL;
-    int32_t nsrmapq = 0;
-    int32_t* srmapq = NULL;
     int32_t nct = 0;
     char* ct = NULL;
     int32_t nsvt = 0;
@@ -128,23 +120,23 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
       if ((bcf_get_info_string(hdr, rec, "SVTYPE", &svt, &nsvt) > 0) && (bcf_get_info_string(hdr, rec, "CT", &ct, &nct) > 0)) recsvt = _decodeOrientation(std::string(ct), std::string(svt));
       if (recsvt != svtin) continue;
 
-      // Correct size
+      // Correct size?
       std::string chrName(bcf_hdr_id2name(hdr, rec->rid));
       uint32_t tid = cMap[chrName];
       uint32_t svStart = rec->pos;
-      uint32_t svEnd = svStart + 1;
+      uint32_t svEnd = rec->pos + 2;
       if (bcf_get_info_int32(hdr, rec, "END", &svend, &nsvend) > 0) svEnd = *svend;
-
-      // SV size check 
-      if (std::string(svt) != "BND") {
-	if (std::string(svt) == "INS") {
-	  unsigned int inslenVal = 0;
-	  if (bcf_get_info_int32(hdr, rec, "INSLEN", &inslen, &ninslen) > 0) inslenVal = *inslen;
-	  if ((inslenVal < c.minsize) || (inslenVal > c.maxsize)) continue;
-	} else {
-	  if ((svEnd - svStart < c.minsize) || (svEnd - svStart > c.maxsize)) continue;
-	}
+      if (recsvt == 4) {
+	// Insertion
+	uint32_t inslenVal = 0;
+	if (bcf_get_info_int32(hdr, rec, "INSLEN", &inslen, &ninslen) > 0) inslenVal = *inslen;
+	if ((inslenVal < c.minsize) || (inslenVal > c.maxsize)) continue;
+      } else {
+	// Other intra-chr SV
+	if ((svEnd - svStart < c.minsize) || (svEnd - svStart > c.maxsize)) continue;
       }
+
+      // Precise?
       bool precise = false;
       if (bcf_get_info_flag(hdr, rec, "PRECISE", 0, 0) > 0) precise=true;
       if ((c.filterForPrecise) && (!precise)) continue;
@@ -192,45 +184,12 @@ void _fillIntervalMap(MergeConfig const& c, TGenomeIntervals& iScore, TContigMap
 	if (gt != NULL) free(gt);
 	if ((maxvaf < c.vaf) || (maxcov < c.coverage)) continue;
       }
-
-      // Get SV support
-      int32_t peSupport = 0;
-      if (bcf_get_info_int32(hdr, rec, "PE", &pe, &npe) > 0) peSupport = *pe;
-      int32_t srSupport = 0;
-      if (bcf_get_info_int32(hdr, rec, "SR", &sr, &nsr) > 0) srSupport = *sr;
-
-      int32_t peMapQuality = 0;
-      if (bcf_get_info_int32(hdr, rec, "MAPQ", &mapq, &nmapq) > 0) peMapQuality = *mapq;
-      int32_t srMapQuality = 0;
-      if (bcf_get_info_int32(hdr, rec, "SRMAPQ", &srmapq, &nsrmapq) > 0) srMapQuality = *srmapq;
-
-      // Quality score for the SV
-      int32_t score = 3 * srSupport * srMapQuality + peSupport * peMapQuality;
-      if (_isKeyPresent(hdr, "SCORE")) {
-	int32_t nvcfscore = 0;
-	if (_getInfoType(hdr, "SCORE") == BCF_HT_INT) {
-	  int32_t* vcfscore = NULL;
-	  bcf_get_info_int32(hdr, rec, "SCORE", &vcfscore, &nvcfscore);
-	  score = *vcfscore;
-	  free(vcfscore);
-	} else if (_getInfoType(hdr, "SCORE") == BCF_HT_REAL) {
-	  float* vcfscore = NULL;
-	  bcf_get_info_float(hdr, rec, "SCORE", &vcfscore, &nvcfscore);
-	  score = (uint32_t) (*vcfscore * 10000); // for scores in [0,1] 
-	  free(vcfscore);
-	}
-      }
-
       // Store the interval
-      iScore[tid].push_back(IntervalScore(svStart, svEnd, score));
+      iScore[tid].push_back(IntervalScore(svStart, svEnd, rec->qual));
     }
     if (svend != NULL) free(svend);
     if (inslen != NULL) free(inslen);
-    if (pe != NULL) free(pe);
-    if (sr != NULL) free(sr);
-    if (mapq != NULL) free(mapq);
     if (ct != NULL) free(ct);
-    if (srmapq != NULL) free(srmapq);
     if (svt != NULL) free(svt);
     bcf_hdr_destroy(hdr);
     bcf_close(ifile);
@@ -301,25 +260,28 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   bcf_hdr_append(hdr_out, "##ALT=<ID=INV,Description=\"Inversion\">");
   bcf_hdr_append(hdr_out, "##ALT=<ID=BND,Description=\"Translocation\">");
   bcf_hdr_append(hdr_out, "##ALT=<ID=INS,Description=\"Insertion\">");
-  bcf_hdr_append(hdr_out, "##FILTER=<ID=LowQual,Description=\"PE/SR support below 3 or mapping quality below 20.\">");
+  bcf_hdr_append(hdr_out, "##FILTER=<ID=LowQual,Description=\"Poor quality and insufficient number of PEs and SRs.\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CIEND,Number=2,Type=Integer,Description=\"PE confidence interval around END\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CIPOS,Number=2,Type=Integer,Description=\"PE confidence interval around POS\">");
-  bcf_hdr_append(hdr_out, "##INFO=<ID=CHR2,Number=1,Type=String,Description=\"Chromosome for END coordinate in case of a translocation\">");
+  bcf_hdr_append(hdr_out, "##INFO=<ID=CHR2,Number=1,Type=String,Description=\"Chromosome for POS2 coordinate in case of an inter-chromosomal translocation\">");
+  bcf_hdr_append(hdr_out, "##INFO=<ID=POS2,Number=1,Type=Integer,Description=\"Genomic position for CHR2 in case of an inter-chromosomal translocation\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=END,Number=1,Type=Integer,Description=\"End position of the structural variant\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=PE,Number=1,Type=Integer,Description=\"Paired-end support of the structural variant\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=MAPQ,Number=1,Type=Integer,Description=\"Median mapping quality of paired-ends\">");
-  bcf_hdr_append(hdr_out, "##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Split-read support\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SRMAPQ,Number=1,Type=Integer,Description=\"Median mapping quality of split-reads\">");
+  bcf_hdr_append(hdr_out, "##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Split-read support\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SRQ,Number=1,Type=Float,Description=\"Split-read consensus alignment quality\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CONSENSUS,Number=1,Type=String,Description=\"Split-read consensus sequence\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CE,Number=1,Type=Float,Description=\"Consensus sequence entropy\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=CT,Number=1,Type=String,Description=\"Paired-end signature induced connection type\">");
+  bcf_hdr_append(hdr_out, "##INFO=<ID=SVLEN,Number=1,Type=Integer,Description=\"Insertion length for SVTYPE=INS.\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=IMPRECISE,Number=0,Type=Flag,Description=\"Imprecise structural variation\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=PRECISE,Number=0,Type=Flag,Description=\"Precise structural variation\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SVTYPE,Number=1,Type=String,Description=\"Type of structural variant\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=SVMETHOD,Number=1,Type=String,Description=\"Type of approach used to detect SV\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=INSLEN,Number=1,Type=Integer,Description=\"Predicted length of the insertion\">");
   bcf_hdr_append(hdr_out, "##INFO=<ID=HOMLEN,Number=1,Type=Integer,Description=\"Predicted microhomology length using a max. edit distance of 2\">");
+  
   // Add reference contigs
   uint32_t numseq = 0;
   typedef std::map<uint32_t, std::string> TReverseMap;
@@ -331,7 +293,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
     bcf_hdr_append(hdr_out, refname.c_str());
   }
   bcf_hdr_add_sample(hdr_out, NULL);
-  bcf_hdr_write(fp, hdr_out);
+  if (bcf_hdr_write(fp, hdr_out) != 0) std::cerr << "Error: Failed to write BCF header!" << std::endl;
 
   // Duplicate filter (identical start, end, score values)
   typedef std::pair<uint32_t, uint32_t> TStartEnd;
@@ -353,7 +315,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
     ifile[file_c] = bcf_open(c.files[file_c].string().c_str(), "r");
     hdr[file_c] = bcf_hdr_read(ifile[file_c]);
-    bcf_hdr_set_samples(hdr[file_c], NULL, false); // Do not read the sample information
+    if (bcf_hdr_set_samples(hdr[file_c], NULL, false) != 0) std::cerr << "Error: Failed to set sample information!" << std::endl;
     rec[file_c] = bcf_init();
     if (bcf_read(ifile[file_c], hdr[file_c], rec[file_c]) == 0) {
       bcf_unpack(rec[file_c], BCF_UN_INFO);
@@ -372,6 +334,8 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   int32_t* sr = NULL;
   int32_t ninslen = 0;
   int32_t* inslen = NULL;
+  int32_t npos2 = 0;
+  int32_t* pos2 = NULL;
   int32_t nhomlen = 0;
   int32_t* homlen = NULL;
   int32_t nmapq = 0;
@@ -442,27 +406,13 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	  int32_t srMapQuality = 0;
 	  if (bcf_get_info_int32(hdr[idx], rec[idx], "SRMAPQ", &srmapq, &nsrmapq) > 0) srMapQuality = *srmapq;
 	  std::string chr2Name = chrName;
+	  int32_t pos2val = 0;
 	  if (bcf_get_info_string(hdr[idx], rec[idx], "CHR2", &chr2, &nchr2) > 0) {
 	    chr2Name = std::string(chr2);
+	    if (bcf_get_info_int32(hdr[idx], rec[idx], "POS2", &pos2, &npos2) > 0) pos2val = *pos2;
 	    //mtid = cMap[chr2Name];
 	  }
-
-	  // Quality score for the SV
-	  int32_t score = 3 * srSupport * srMapQuality + peSupport * peMapQuality;
-	  if (_isKeyPresent(hdr[idx], "SCORE")) {
-	    int32_t nvcfscore = 0;
-	    if (_getInfoType(hdr[idx], "SCORE") == BCF_HT_INT) {
-	      int32_t* vcfscore = NULL;
-	      bcf_get_info_int32(hdr[idx], rec[idx], "SCORE", &vcfscore, &nvcfscore);
-	      score = *vcfscore;
-	      free(vcfscore);
-	    } else if (_getInfoType(hdr[idx], "SCORE") == BCF_HT_REAL) {
-	      float* vcfscore = NULL;
-	      bcf_get_info_float(hdr[idx], rec[idx], "SCORE", &vcfscore, &nvcfscore);
-	      score = (uint32_t) (*vcfscore * 10000); // for scores in [0,1] 
-	      free(vcfscore);
-	    }
-	  }
+	  int32_t score = rec[idx]->qual;
 	  
 	  // Is this a selected interval
 	  typename TIntervalScores::const_iterator iter = std::lower_bound(iSelected[tid].begin(), iSelected[tid].end(), IntervalScore(svStart, svEnd, score), SortIScores<IntervalScore>());
@@ -490,14 +440,13 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    float ceVal = 0;
 	    if (precise) {
 	      if (bcf_get_info_float(hdr[idx], rec[idx], "CE", &ce, &nce) > 0) ceVal = *ce;
-	      bcf_get_info_string(hdr[idx], rec[idx], "CONSENSUS", &cons, &ncons);
-	      consensus = boost::to_upper_copy(std::string(cons));
+	      if (bcf_get_info_string(hdr[idx], rec[idx], "CONSENSUS", &cons, &ncons) > 0) consensus = boost::to_upper_copy(std::string(cons));
 	    }
 	    
 	    // Create new record
 	    rout->rid = bcf_hdr_name2id(hdr_out, chrName.c_str());
 	    rout->pos = rec[idx]->pos;
-	    rout->qual = score;
+	    rout->qual = rec[idx]->qual;
 	    std::string id;
 	    if (c.files.size() == 1) id = std::string(rec[idx]->d.id); // Within one VCF file IDs are unique
 	    else {
@@ -521,8 +470,14 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    std::string dellyVersion("EMBL.DELLYv");
 	    dellyVersion += dellyVersionNumber;
 	    bcf_update_info_string(hdr_out,rout, "SVMETHOD", dellyVersion.c_str());
-	    bcf_update_info_string(hdr_out,rout, "CHR2", chr2Name.c_str());
 	    bcf_update_info_int32(hdr_out, rout, "END", &svEnd, 1);
+	    if (svtin >= DELLY_SVT_TRANS) {
+	      bcf_update_info_string(hdr_out,rout, "CHR2", chr2Name.c_str());
+	      bcf_update_info_int32(hdr_out, rout, "POS2", &pos2val, 1);
+	    }
+	    if (svtin == 4) {
+	      bcf_update_info_int32(hdr_out, rout, "SVLEN", &inslenVal, 1);
+	    }
 	    bcf_update_info_int32(hdr_out, rout, "PE", &peSupport, 1);
 	    int32_t tmpi = peMapQuality;
 	    bcf_update_info_int32(hdr_out, rout, "MAPQ", &tmpi, 1);
@@ -530,14 +485,16 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
 	    bcf_update_info_int32(hdr_out, rout, "CIPOS", cipos, 2);
 	    bcf_update_info_int32(hdr_out, rout, "CIEND", ciend, 2);
 	    if (precise) {
+	      int32_t tmpi = srMapQuality;
+	      bcf_update_info_int32(hdr_out, rout, "SRMAPQ", &tmpi, 1);
 	      bcf_update_info_int32(hdr_out, rout, "INSLEN", &inslenVal, 1);
 	      bcf_update_info_int32(hdr_out, rout, "HOMLEN", &homlenVal, 1);
 	      bcf_update_info_int32(hdr_out, rout, "SR", &srSupport, 1);
-	      int32_t tmpi = srMapQuality;
-	      bcf_update_info_int32(hdr_out, rout, "SRMAPQ", &tmpi, 1);
 	      bcf_update_info_float(hdr_out, rout, "SRQ", &srAlignQuality, 1);
-	      bcf_update_info_string(hdr_out, rout, "CONSENSUS", consensus.c_str());
-	      bcf_update_info_float(hdr_out, rout, "CE", &ceVal, 1);
+	      if (consensus.size()) {
+		bcf_update_info_string(hdr_out, rout, "CONSENSUS", consensus.c_str());
+		bcf_update_info_float(hdr_out, rout, "CE", &ceVal, 1);
+	      }
 	    }
 	
 	    // Write record
@@ -561,6 +518,7 @@ void _outputSelectedIntervals(MergeConfig& c, TGenomeIntervals const& iSelected,
   if (sr != NULL) free(sr);
   if (homlen != NULL) free(homlen);
   if (inslen != NULL) free(inslen);
+  if (pos2 != NULL) free(pos2);
   if (mapq != NULL) free(mapq);
   if (srmapq != NULL) free(srmapq);
   if (ct != NULL) free(ct);
@@ -620,7 +578,7 @@ mergeBCFs(MergeConfig& c, std::vector<boost::filesystem::path> const& cts) {
   // Open output VCF file
   htsFile *fp = hts_open(c.outfile.string().c_str(), "wb");
   bcf_hdr_t *hdr_out = bcf_hdr_dup(hdr[0]);
-  bcf_hdr_write(fp, hdr_out);
+  if (bcf_hdr_write(fp, hdr_out) != 0) std::cerr << "Error: Failed to write BCF header!" << std::endl;
 
   // Merge files
   while (allEOF < cts.size()) {
@@ -755,7 +713,7 @@ int merge(int argc, char **argv) {
   // Check command line arguments
   if ((vm.count("help")) || (!vm.count("input-file"))) { 
     std::cout << std::endl;
-    std::cout << "Usage: delly " << argv[0] << " [OPTIONS] [<sample1.bcf> <sample2.bcf> ... | <list_of_bcf_files.tsv>]" << std::endl;
+    std::cout << "Usage: delly " << argv[0] << " [OPTIONS] [<sample1.bcf> <sample2.bcf> ... | <list_of_bcf_files.txt>]" << std::endl;
     std::cout << visible_options << "\n"; 
     return 0; 
   }
@@ -780,26 +738,37 @@ int merge(int argc, char **argv) {
 
   // Check input BCF files
   if (c.files.size() == 1) {
-    // Assume the input file is a list
+    // Single file: VCF/BCF or list of files?
     if (!(boost::filesystem::exists(c.files[0]) && boost::filesystem::is_regular_file(c.files[0]) && boost::filesystem::file_size(c.files[0]))) {
       std::cerr << "Input file list " << c.files[0].string() << " is missing!" << std::endl;
       return 1;
     }
-    std::string fname = c.files[0].string();
-    c.files.clear();
-    std::ifstream lf(fname.c_str());
-    if (lf.good()) {
-      std::string line;
-      while(std::getline(lf, line)) {
-	if (!line.empty()) {
-	  if (line.at(line.length() - 1) == '\r' ) {
-	    line = line.substr(0, line.length() - 1);
+    htsFile* inf = bcf_open(c.files[0].string().c_str(), "r");
+    if (inf != NULL) {
+      bcf_hdr_t* header = NULL;
+      header = bcf_hdr_read(inf);
+      if (header == NULL) {
+	std::cout << "Assuming input is a list of BCF files" << std::endl;
+	std::string fname = c.files[0].string();
+	c.files.clear();
+	std::ifstream lf(fname.c_str());
+	if (lf.good()) {
+	  std::string line;
+	  while(std::getline(lf, line)) {
+	    if (!line.empty()) {
+	      if (line.at(line.length() - 1) == '\r' ) {
+		line = line.substr(0, line.length() - 1);
+	      }
+	      c.files.push_back(line);
+	    }
 	  }
-	  c.files.push_back(line);
+	  lf.close();
 	}
+      } else {
+	bcf_hdr_destroy(header);
       }
-      lf.close();
     }
+    bcf_close(inf);
   }
   for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) {
     htsFile* ifile = bcf_open(c.files[file_c].string().c_str(), "r");
