@@ -23,7 +23,7 @@ void _remove_format_tag(bcf_hdr_t* hdr, bcf1_t* rec, std::string const& tag) {
 }
 
 void _remove_info(bcf_hdr_t* hdr, bcf1_t* rec) {
-  std::string tmp[] = {"CT", "PRECISE", "IMPRECISE", "SVTYPE", "SVMETHOD", "CIEND", "CIPOS", "CHR2", "END", "PE", "MAPQ", "SRMAPQ", "SR", "SRQ", "CONSENSUS"};
+  std::string tmp[] = {"CT", "PRECISE", "IMPRECISE", "SVTYPE", "SVMETHOD", "CIEND", "CIPOS", "CHR2", "POS2", "END", "PE", "MAPQ", "SRMAPQ", "SR", "SRQ", "CONSENSUS"};
   std::set<std::string> keepInfo(tmp, tmp + sizeof(tmp)/sizeof(tmp[0]));
 
   if (!(rec->unpacked & BCF_UN_INFO)) bcf_unpack(rec, BCF_UN_INFO);
@@ -165,6 +165,10 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
   // Parse bcf
   int32_t nsvend = 0;
   int32_t* svend = NULL;
+  int32_t npos2 = 0;
+  int32_t* pos2 = NULL;
+  int32_t nsvlen = 0;
+  int32_t* svlen = NULL;
   int32_t npe = 0;
   int32_t* pe = NULL;
   int32_t ninslen = 0;
@@ -211,10 +215,15 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
       svRec.chr = tid;
       svRec.svStart = rec->pos + 1;
       svRec.id = svs.size();
+      svRec.mapq = rec->qual;
       std::string refAllele = rec->d.allele[0];
       std::string altAllele = rec->d.allele[1];
       svRec.alleles = refAllele + "," + altAllele;
 
+      // Parse SV type
+      if ((bcf_get_info_string(hdr, rec, "SVTYPE", &svt, &nsvt) > 0) && (bcf_get_info_string(hdr, rec, "CT", &ct, &nct) > 0)) svRec.svt = _decodeOrientation(std::string(ct), std::string(svt));
+      else continue;
+      
       // Parse INFO
       if (bcf_get_info_flag(hdr, rec, "PRECISE", 0, 0) > 0) svRec.precise=true;
       else svRec.precise = false;
@@ -223,14 +232,33 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
 	if (svRec.precise) svRec.peSupport = 0;
 	else svRec.peSupport = 2;
       }
-      if (bcf_get_info_int32(hdr, rec, "INSLEN", &inslen, &ninslen) > 0) svRec.insLen = *inslen;
-      else svRec.insLen = 0;
+      if (svRec.svt != 4) {
+	if (bcf_get_info_int32(hdr, rec, "INSLEN", &inslen, &ninslen) > 0) svRec.insLen = *inslen;
+	else svRec.insLen = 0;
+      } else {
+	// Insertions must have INFO/SVLEN
+	if (bcf_get_info_int32(hdr, rec, "SVLEN", &svlen, &nsvlen) > 0) svRec.insLen = *svlen;
+	else continue;
+      }
       if (bcf_get_info_int32(hdr, rec, "HOMLEN", &homlen, &nhomlen) > 0) svRec.homLen = *homlen;
       else svRec.homLen = 0;
       if (bcf_get_info_int32(hdr, rec, "SR", &sr, &nsr) > 0) svRec.srSupport = *sr;
       else svRec.srSupport = 0;
-      if (bcf_get_info_int32(hdr, rec, "END", &svend, &nsvend) > 0) svRec.svEnd = *svend;
-      else svRec.svEnd = rec->pos + 2;
+
+      // SV end assignment
+      svRec.chr2 = tid;
+      svRec.svEnd = rec->pos + 2;
+      if (svRec.svt < DELLY_SVT_TRANS) {
+	// Intra-chromosomal SV
+	if (bcf_get_info_int32(hdr, rec, "END", &svend, &nsvend) > 0) svRec.svEnd = *svend;
+      } else {
+	// Inter-chromosomal SV
+	if (bcf_get_info_string(hdr, rec, "CHR2", &chr2, &nchr2) > 0) {
+	  std::string chr2Name = std::string(chr2);
+	  svRec.chr2 = bam_name2id(hd, chr2Name.c_str());
+	}
+	if (bcf_get_info_int32(hdr, rec, "POS2", &pos2, &npos2) > 0) svRec.svEnd = *pos2;
+      }
       if (bcf_get_info_string(hdr, rec, "CONSENSUS", &cons, &ncons) > 0) svRec.consensus = std::string(cons);
       else svRec.precise = false;
       if (bcf_get_info_int32(hdr, rec, "CIPOS", &cipos, &ncipos) > 0) {
@@ -253,12 +281,7 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
       else svRec.srMapQuality = 0;
       if (bcf_get_info_float(hdr, rec, "SRQ", &srq, &nsrq) > 0) svRec.srAlignQuality = (double) *srq;
       else svRec.srAlignQuality = 0;
-      if ((bcf_get_info_string(hdr, rec, "SVTYPE", &svt, &nsvt) > 0) && (bcf_get_info_string(hdr, rec, "CT", &ct, &nct) > 0)) svRec.svt = _decodeOrientation(std::string(ct), std::string(svt));
-      else continue;
-      if (bcf_get_info_string(hdr, rec, "CHR2", &chr2, &nchr2) > 0) {
-	std::string chr2Name = std::string(chr2);
-	svRec.chr2 = bam_name2id(hd, chr2Name.c_str());
-      } else svRec.chr2 = tid;
+
       svs.push_back(svRec);
     } else if (wimethod == 2) {
       // Assume precise SV, only deletions supported and INFO:END is required!!!
@@ -349,6 +372,8 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
   }
   // Clean-up
   free(svend);
+  free(pos2);
+  free(svlen);
   free(svt);
   free(method);
   free(pe);
@@ -373,148 +398,6 @@ vcfParse(TConfig const& c, bam_hdr_t* hd, std::vector<TStructuralVariantRecord>&
 }
 
 
-// Long-reads
-template<typename TConfig>
-inline void
-outputVcf(TConfig const& c, std::vector<StructuralVariantRecord> const& sv)
-{
-  typedef std::vector<StructuralVariantRecord> TSVs;
-  
-  // Open one bam file header
-  samFile* samfile = sam_open(c.files[0].string().c_str(), "r");  
-  hts_set_fai_filename(samfile, c.genome.string().c_str());
-  bam_hdr_t* bamhd = sam_hdr_read(samfile);
-
-  // Output all structural variants
-  htsFile *fp = hts_open(c.outfile.string().c_str(), "wb");
-  bcf_hdr_t *hdr = bcf_hdr_init("w");
-
-  // Print vcf header
-  boost::posix_time::ptime now = boost::posix_time::second_clock::local_time();
-  boost::gregorian::date today = now.date();
-  std::string datestr("##fileDate=");
-  datestr += boost::gregorian::to_iso_string(today);
-  bcf_hdr_append(hdr, datestr.c_str());
-  bcf_hdr_append(hdr, "##ALT=<ID=DEL,Description=\"Deletion\">");
-  bcf_hdr_append(hdr, "##ALT=<ID=DUP,Description=\"Duplication\">");
-  bcf_hdr_append(hdr, "##ALT=<ID=INV,Description=\"Inversion\">");
-  bcf_hdr_append(hdr, "##ALT=<ID=BND,Description=\"Translocation\">");
-  bcf_hdr_append(hdr, "##ALT=<ID=INS,Description=\"Insertion\">");
-  bcf_hdr_append(hdr, "##FILTER=<ID=LowQual,Description=\"Long read support below 3.\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=CIPOS,Number=2,Type=Integer,Description=\"Breakpoint confidence interval around POS\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=CIEND,Number=2,Type=Integer,Description=\"Breakpoint confidence interval around END\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=CHR2,Number=1,Type=String,Description=\"Chromosome for END coordinate in case of a translocation\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=END,Number=1,Type=Integer,Description=\"End position of the structural variant\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=SVLEN,Number=1,Type=Integer,Description=\"SV length\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=SR,Number=1,Type=Integer,Description=\"Long read support\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=SRQ,Number=1,Type=Float,Description=\"Split-read consensus alignment quality\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=MAPQ,Number=1,Type=Integer,Description=\"Mean mapping quality of long reads\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=CT,Number=1,Type=String,Description=\"Connection type\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=SVTYPE,Number=1,Type=String,Description=\"Type of structural variant\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=SVMETHOD,Number=1,Type=String,Description=\"Type of approach used to detect SV\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=INSLEN,Number=1,Type=Integer,Description=\"Predicted length of the insertion\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=CONSENSUS,Number=1,Type=String,Description=\"Split-read consensus sequence\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=CE,Number=1,Type=Float,Description=\"Consensus sequence entropy\">");
-
-  // Add reference
-  std::string refloc("##reference=");
-  refloc += c.genome.string();
-  bcf_hdr_append(hdr, refloc.c_str());
-  for (int i = 0; i<bamhd->n_targets; ++i) {
-    std::string refname("##contig=<ID=");
-    refname += std::string(bamhd->target_name[i]) + ",length=" + boost::lexical_cast<std::string>(bamhd->target_len[i]) + ">";
-    bcf_hdr_append(hdr, refname.c_str());
-  }
-  // Add samples
-  bcf_hdr_add_sample(hdr, NULL);
-  bcf_hdr_write(fp, hdr);
-
-  if (!sv.empty()) {
-    // Quality threshold estimation
-    std::vector<float> qcf;
-    for(typename TSVs::const_iterator svIter = sv.begin(); svIter!=sv.end(); ++svIter) {
-      if (svIter->srSupport == 0) continue;
-      qcf.push_back((float) svIter->srSupport * (float) svIter->srAlignQuality);
-    }
-    std::sort(qcf.begin(), qcf.end());
-    float qualThreshold = qcf[qcf.size() * 0.05];
-    
-    // Iterate SVs
-    bcf1_t *rec = bcf_init();
-    for(typename TSVs::const_iterator svIter = sv.begin(); svIter!=sv.end(); ++svIter) {
-      if (svIter->srSupport == 0) continue;
-
-      // Output main vcf fields
-      float qcv = (float) svIter->srSupport * svIter->srAlignQuality;
-      int32_t tmpi = bcf_hdr_id2int(hdr, BCF_DT_ID, "PASS");
-      if (qcv < qualThreshold) tmpi = bcf_hdr_id2int(hdr, BCF_DT_ID, "LowQual");
-      rec->rid = bcf_hdr_name2id(hdr, bamhd->target_name[svIter->chr]);
-      rec->pos = svIter->svStart;
-      std::string id(_addID(svIter->svt));
-      std::string padNumber = boost::lexical_cast<std::string>(svIter->id);
-      padNumber.insert(padNumber.begin(), 8 - padNumber.length(), '0');
-      id += padNumber;
-      bcf_update_id(hdr, rec, id.c_str());
-      std::string alleles = "N,<";
-      alleles += _addID(svIter->svt);
-      alleles += ">";
-      bcf_update_alleles_str(hdr, rec, alleles.c_str());
-      bcf_update_filter(hdr, rec, &tmpi, 1);
-      
-      // Add INFO fields
-      bcf_update_info_string(hdr, rec, "SVTYPE", _addID(svIter->svt).c_str());
-      std::string dellyVersion("EMBL.DELLYv");
-      dellyVersion += dellyVersionNumber;
-      bcf_update_info_string(hdr,rec, "SVMETHOD", dellyVersion.c_str());
-      bcf_update_info_string(hdr,rec, "CHR2", bamhd->target_name[svIter->chr2]);
-      if (svIter->svt != 4) tmpi = svIter->svEnd;
-      else tmpi = svIter->svStart + 2;
-      bcf_update_info_int32(hdr, rec, "END", &tmpi, 1);
-      if (svIter->svt < DELLY_SVT_TRANS) {
-	if (svIter->svt != 4) tmpi = svIter->svEnd - svIter->svStart;
-	else tmpi = svIter->insLen;
-      } else tmpi = 0;
-      bcf_update_info_int32(hdr, rec, "SVLEN", &tmpi, 1);
-      bcf_update_info_string(hdr, rec, "CT", _addOrientation(svIter->svt).c_str());
-      int32_t ciend[2];
-      ciend[0] = svIter->ciendlow;
-      ciend[1] = svIter->ciendhigh;
-      int32_t cipos[2];
-      cipos[0] = -svIter->ciposlow;
-      cipos[1] = svIter->ciposhigh;
-      bcf_update_info_int32(hdr, rec, "CIPOS", cipos, 2);
-      bcf_update_info_int32(hdr, rec, "CIEND", ciend, 2);
-      tmpi = svIter->srSupport;
-      bcf_update_info_int32(hdr, rec, "SR", &tmpi, 1);
-      tmpi = (int32_t) (svIter->mapq / svIter->srSupport);
-      bcf_update_info_int32(hdr, rec, "MAPQ", &tmpi, 1);
-      tmpi = svIter->insLen;
-      bcf_update_info_int32(hdr, rec, "INSLEN", &tmpi, 1);
-      float tmpf = svIter->srAlignQuality;
-      bcf_update_info_float(hdr, rec, "SRQ", &tmpf, 1);
-      bcf_update_info_string(hdr, rec, "CONSENSUS", svIter->consensus.c_str());
-      tmpf = entropy(svIter->consensus);
-      bcf_update_info_float(hdr, rec, "CE", &tmpf, 1);
-      
-      rec->qual = svIter->mapq;
-      bcf_write1(fp, hdr, rec);
-      bcf_clear1(rec);
-    }
-    bcf_destroy1(rec);
-  }
-
-  // Close BAM file
-  bam_hdr_destroy(bamhd);
-  sam_close(samfile);
-
-  // Close VCF file
-  bcf_hdr_destroy(hdr);
-  hts_close(fp);
-
-  // Build index
-  bcf_index_build(c.outfile.string().c_str(), 14);
-}
- 
 template<typename TConfig, typename TStructuralVariantRecord, typename TJunctionCountMap, typename TReadCountMap, typename TCountMap>
 inline void
 vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJunctionCountMap const& jctCountMap, TReadCountMap const& readCountMap, TCountMap const& spanCountMap)
@@ -545,7 +428,8 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
   bcf_hdr_append(hdr, "##FILTER=<ID=LowQual,Description=\"Poor quality and insufficient number of PEs and SRs.\">");
   bcf_hdr_append(hdr, "##INFO=<ID=CIEND,Number=2,Type=Integer,Description=\"PE confidence interval around END\">");
   bcf_hdr_append(hdr, "##INFO=<ID=CIPOS,Number=2,Type=Integer,Description=\"PE confidence interval around POS\">");
-  bcf_hdr_append(hdr, "##INFO=<ID=CHR2,Number=1,Type=String,Description=\"Chromosome for END coordinate in case of a translocation\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=CHR2,Number=1,Type=String,Description=\"Chromosome for POS2 coordinate in case of an inter-chromosomal translocation\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=POS2,Number=1,Type=Integer,Description=\"Genomic position for CHR2 in case of an inter-chromosomal translocation\">");
   bcf_hdr_append(hdr, "##INFO=<ID=END,Number=1,Type=Integer,Description=\"End position of the structural variant\">");
   bcf_hdr_append(hdr, "##INFO=<ID=PE,Number=1,Type=Integer,Description=\"Paired-end support of the structural variant\">");
   bcf_hdr_append(hdr, "##INFO=<ID=MAPQ,Number=1,Type=Integer,Description=\"Median mapping quality of paired-ends\">");
@@ -555,6 +439,7 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
   bcf_hdr_append(hdr, "##INFO=<ID=CONSENSUS,Number=1,Type=String,Description=\"Split-read consensus sequence\">");
   bcf_hdr_append(hdr, "##INFO=<ID=CE,Number=1,Type=Float,Description=\"Consensus sequence entropy\">");
   bcf_hdr_append(hdr, "##INFO=<ID=CT,Number=1,Type=String,Description=\"Paired-end signature induced connection type\">");
+  bcf_hdr_append(hdr, "##INFO=<ID=SVLEN,Number=1,Type=Integer,Description=\"Insertion length for SVTYPE=INS.\">");
   bcf_hdr_append(hdr, "##INFO=<ID=IMPRECISE,Number=0,Type=Flag,Description=\"Imprecise structural variation\">");
   bcf_hdr_append(hdr, "##INFO=<ID=PRECISE,Number=0,Type=Flag,Description=\"Precise structural variation\">");
   bcf_hdr_append(hdr, "##INFO=<ID=SVTYPE,Number=1,Type=String,Description=\"Type of structural variant\">");
@@ -565,9 +450,9 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
   bcf_hdr_append(hdr, "##FORMAT=<ID=GL,Number=G,Type=Float,Description=\"Log10-scaled genotype likelihoods for RR,RA,AA genotypes\">");
   bcf_hdr_append(hdr, "##FORMAT=<ID=GQ,Number=1,Type=Integer,Description=\"Genotype Quality\">");
   bcf_hdr_append(hdr, "##FORMAT=<ID=FT,Number=1,Type=String,Description=\"Per-sample genotype filter\">");
-  bcf_hdr_append(hdr, "##FORMAT=<ID=RC,Number=1,Type=Integer,Description=\"Raw high-quality read counts for the SV\">");
-  bcf_hdr_append(hdr, "##FORMAT=<ID=RCL,Number=1,Type=Integer,Description=\"Raw high-quality read counts for the left control region\">");
-  bcf_hdr_append(hdr, "##FORMAT=<ID=RCR,Number=1,Type=Integer,Description=\"Raw high-quality read counts for the right control region\">");
+  bcf_hdr_append(hdr, "##FORMAT=<ID=RC,Number=1,Type=Integer,Description=\"Raw high-quality read counts or base counts for the SV\">");
+  bcf_hdr_append(hdr, "##FORMAT=<ID=RCL,Number=1,Type=Integer,Description=\"Raw high-quality read counts or base counts for the left control region\">");
+  bcf_hdr_append(hdr, "##FORMAT=<ID=RCR,Number=1,Type=Integer,Description=\"Raw high-quality read counts or base counts for the right control region\">");
   bcf_hdr_append(hdr, "##FORMAT=<ID=CN,Number=1,Type=Integer,Description=\"Read-depth based copy-number estimate for autosomal sites\">");
   bcf_hdr_append(hdr, "##FORMAT=<ID=DR,Number=1,Type=Integer,Description=\"# high-quality reference pairs\">");
   bcf_hdr_append(hdr, "##FORMAT=<ID=DV,Number=1,Type=Integer,Description=\"# high-quality variant pairs\">");
@@ -597,7 +482,7 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
   // Add samples
   for(unsigned int file_c = 0; file_c < c.files.size(); ++file_c) bcf_hdr_add_sample(hdr, c.sampleName[file_c].c_str());
   bcf_hdr_add_sample(hdr, NULL);
-  bcf_hdr_write(fp, hdr);
+  if (bcf_hdr_write(fp, hdr) != 0) std::cerr << "Error: Failed to write BCF header!" << std::endl;
 
   if (!svs.empty()) {
     // Genotype arrays
@@ -631,6 +516,8 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
     bcf1_t *rec = bcf_init();
     for(typename TSVs::const_iterator svIter = svs.begin(); svIter!=svs.end(); ++svIter) {
       ++show_progress;
+      if ((svIter->srSupport == 0) && (svIter->peSupport == 0)) continue;
+      
       // Output main vcf fields
       int32_t tmpi = bcf_hdr_id2int(hdr, BCF_DT_ID, "PASS");
       if (svIter->chr == svIter->chr2) {
@@ -663,9 +550,20 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
       std::string dellyVersion("EMBL.DELLYv");
       dellyVersion += dellyVersionNumber;
       bcf_update_info_string(hdr,rec, "SVMETHOD", dellyVersion.c_str());
-      bcf_update_info_string(hdr,rec, "CHR2", bamhd->target_name[svIter->chr2]);
-      tmpi = svEndPos;
-      bcf_update_info_int32(hdr, rec, "END", &tmpi, 1);
+      if (svIter->svt < DELLY_SVT_TRANS) {
+	tmpi = svEndPos;
+	bcf_update_info_int32(hdr, rec, "END", &tmpi, 1);
+      } else {
+	tmpi = svStartPos + 2;
+	bcf_update_info_int32(hdr, rec, "END", &tmpi, 1);
+	bcf_update_info_string(hdr,rec, "CHR2", bamhd->target_name[svIter->chr2]);
+	tmpi = svEndPos;
+	bcf_update_info_int32(hdr, rec, "POS2", &tmpi, 1);
+      }
+      if (svIter->svt == 4) {
+	tmpi = svIter->insLen;
+	bcf_update_info_int32(hdr, rec, "SVLEN", &tmpi, 1);
+      }
       tmpi = svIter->peSupport;
       bcf_update_info_int32(hdr, rec, "PE", &tmpi, 1);
       tmpi = svIter->peMapQuality;
@@ -691,9 +589,11 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
 	bcf_update_info_int32(hdr, rec, "SR", &tmpi, 1);
 	float tmpf = svIter->srAlignQuality;
 	bcf_update_info_float(hdr, rec, "SRQ", &tmpf, 1);
-	bcf_update_info_string(hdr, rec, "CONSENSUS", svIter->consensus.c_str());
-	tmpf = entropy(svIter->consensus);
-	bcf_update_info_float(hdr, rec, "CE", &tmpf, 1);
+	if (svIter->consensus.size()) {
+	  bcf_update_info_string(hdr, rec, "CONSENSUS", svIter->consensus.c_str());
+	  tmpf = entropy(svIter->consensus);
+	  bcf_update_info_float(hdr, rec, "CE", &tmpf, 1);
+	}
       }
       
       // Add genotype columns
@@ -751,8 +651,7 @@ vcfOutput(TConfig const& c, std::vector<TStructuralVariantRecord> const& svs, TJ
 	if (gqval[file_c] < 15) ftarr[file_c] = "LowQual";
 	else ftarr[file_c] = "PASS";
       }
-      // ToDo
-      //rec->qual = 0;
+      rec->qual = svIter->mapq;
 
       
       bcf_update_genotypes(hdr, rec, gts, bcf_hdr_nsamples(hdr) * 2);
