@@ -41,6 +41,8 @@ namespace torali
   };
 
 
+
+  
   // Decode Orientation
   inline int32_t
     _decodeOrientation(std::string const& value) {
@@ -81,6 +83,11 @@ namespace torali
     else return "BND";
   }
 
+  inline std::string
+  _addAlleles(std::string const& ref, std::string const& alt) {
+    return ref + "," + alt;
+  }
+      
   inline std::string
   _addAlleles(std::string const& ref, std::string const& chr2, StructuralVariantRecord const& sv, int32_t const svt) {
     if (_translocation(svt)) {
@@ -111,6 +118,73 @@ namespace torali
     else return "NtoN";
   }
 
+  // Output directory/file checks
+  inline bool
+  _outfileValid(boost::filesystem::path& outfile) {
+    try {
+      boost::filesystem::path outdir;
+      if (outfile.has_parent_path()) outdir = outfile.parent_path();
+      else outdir = boost::filesystem::current_path();
+      if (!boost::filesystem::exists(outdir)) {
+	std::cerr << "Output directory does not exist: " << outdir << std::endl;
+	return false;
+      } else {
+	boost::filesystem::file_status s = boost::filesystem::status(outdir);
+	boost::filesystem::ofstream file(outfile.string());
+	file.close();
+	if (!(boost::filesystem::exists(outfile) && boost::filesystem::is_regular_file(outfile))) {
+	  std::cerr << "Fail to open output file " << outfile.string() << std::endl;
+	  std::cerr << "Output directory permissions: " << s.permissions() << std::endl;
+	  return false;
+	} else {
+	  boost::filesystem::remove(outfile.string());
+	}
+      }
+    } catch (boost::filesystem::filesystem_error const& e) {
+      std::cerr << e.what() << std::endl;
+      return false;
+    }
+    return true;
+  }
+
+  template<typename TConfig>
+  inline void
+    _svTypesToCompute(TConfig& c, std::string const& svtype, bool const specified) {
+    c.svtcmd = false;
+    if (specified) {
+      c.svtcmd = true;
+      if (svtype == "DEL") {
+	c.svtset.insert(2);
+      } else if (svtype == "INS") {
+	c.svtset.insert(4);
+      } else if (svtype == "DUP") {
+	c.svtset.insert(3);
+      } else if (svtype == "INV") {
+	c.svtset.insert(0);
+	c.svtset.insert(1);
+      } else if (svtype == "INV_3to3") {
+	c.svtset.insert(0);
+      } else if (svtype == "INV_5to5") {
+	c.svtset.insert(1);
+      } else if (svtype == "BND") {
+	c.svtset.insert(DELLY_SVT_TRANS + 0);
+	c.svtset.insert(DELLY_SVT_TRANS + 1);
+	c.svtset.insert(DELLY_SVT_TRANS + 2);
+	c.svtset.insert(DELLY_SVT_TRANS + 3);
+      } else if (svtype == "BND_3to3") {
+	c.svtset.insert(DELLY_SVT_TRANS + 0);
+      } else if (svtype == "BND_5to5") {
+	c.svtset.insert(DELLY_SVT_TRANS + 1);
+      } else if (svtype == "BND_3to5") {
+	c.svtset.insert(DELLY_SVT_TRANS + 2);
+      } else if (svtype == "BND_5to3") {
+	c.svtset.insert(DELLY_SVT_TRANS + 3);
+      } else {
+	c.svtcmd = false;
+      }
+    }
+  }
+
   inline uint32_t sequenceLength(bam1_t const* rec) {
     uint32_t* cigar = bam_get_cigar(rec);
     uint32_t slen = 0;
@@ -118,7 +192,13 @@ namespace torali
       if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF) || (bam_cigar_op(cigar[i]) == BAM_CINS) || (bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) || (bam_cigar_op(cigar[i]) == BAM_CHARD_CLIP)) slen += bam_cigar_oplen(cigar[i]);
     return slen;
   }
-  
+
+  inline int32_t
+  readLength(bam1_t const* rec) {
+    //int32_t slen = rec->core.l_qseq;  # Incorrect for seq. with hard-clips
+    return sequenceLength(rec);
+  }
+    
   inline uint32_t alignmentLength(bam1_t* rec) {
     uint32_t* cigar = bam_get_cigar(rec);
     uint32_t alen = 0;
@@ -253,6 +333,81 @@ namespace torali
     }
   }
 
+  template<typename TConfig, typename TRegionsGenome>
+  inline int32_t
+  _parseExcludeIntervals(TConfig const& c, bam_hdr_t* hdr, TRegionsGenome& validRegions) {
+    typedef typename TRegionsGenome::value_type TChrIntervals;
+    typedef typename TChrIntervals::interval_type TIVal;
+    
+    validRegions.resize(hdr->n_targets);
+    TRegionsGenome exclg;
+    exclg.resize(hdr->n_targets);
+    std::vector<bool> validChr;
+    validChr.resize(hdr->n_targets, true);
+    if (c.hasExcludeFile) {
+      std::ifstream chrFile(c.exclude.string().c_str(), std::ifstream::in);
+      if (chrFile.is_open()) {
+	while (chrFile.good()) {
+	  std::string chrFromFile;
+	  getline(chrFile, chrFromFile);
+	  typedef boost::tokenizer< boost::char_separator<char> > Tokenizer;
+	  boost::char_separator<char> sep(" \t,;");
+	  Tokenizer tokens(chrFromFile, sep);
+	  Tokenizer::iterator tokIter = tokens.begin();
+	  if (tokIter!=tokens.end()) {
+	    std::string chrName = *tokIter++;
+	    int32_t tid = bam_name2id(hdr, chrName.c_str());
+	    if (tid >= 0) {
+	      if (tokIter!=tokens.end()) {
+		int32_t start = 0;
+		try {
+		  start = boost::lexical_cast<int32_t>(*tokIter++);
+		} catch (boost::bad_lexical_cast&) {
+		  std::cerr << "Exclude file needs to be in tab-delimited format: chr, start, end" << std::endl;
+		  std::cerr << "Offending line: " << chrFromFile << std::endl;
+		  return false;
+		}
+		if (tokIter!=tokens.end()) {
+		  int32_t end = start + 1;
+		  try {
+		    end = boost::lexical_cast<int32_t>(*tokIter++);
+		  } catch (boost::bad_lexical_cast&) {
+		    std::cerr << "Exclude file needs to be in tab-delimited format: chr, start, end" << std::endl;
+		    std::cerr << "Offending line: " << chrFromFile << std::endl;
+		    return false;
+		  }
+		  if (start < end) {
+		    exclg[tid].insert(TIVal::right_open(start, end));
+		  } else {
+		    std::cerr << "Exclude file needs to be in tab-delimited format (chr, start, end) and start < end." << std::endl;
+		    std::cerr << "Offending line: " << chrFromFile << std::endl;
+		    return false;
+		  }
+		} else {
+		  std::cerr << "Exclude file needs to be in tab-delimited format: chr, start, end" << std::endl;
+		  std::cerr << "Offending line: " << chrFromFile << std::endl;
+		  return false;
+		}
+	      } else validChr[tid] = false; // Exclude entire chromosome
+	    }
+	  }
+	}
+	chrFile.close();
+      }
+    }
+    // Create the valid regions
+    for (int32_t i = 0; i<hdr->n_targets; ++i) {
+      if (!validChr[i]) continue;
+      uint32_t istart = 0;
+      for(typename TChrIntervals::iterator it = exclg[i].begin(); it != exclg[i].end(); ++it) {
+	if (istart + 1 < it->lower()) validRegions[i].insert(TIVal::right_open(istart, it->lower() - 1));
+	istart = it->upper();
+      }
+      if (istart + 1 < hdr->target_len[i]) validRegions[i].insert(TIVal::right_open(istart, hdr->target_len[i]));
+    }
+    exclg.clear();
+    return true;
+  }
 
   template<typename TIterator, typename TValue>
   inline void
@@ -434,6 +589,30 @@ namespace torali
   }
 
 
+  template<typename TAlign>
+  inline uint32_t
+  _trimAlignedSequences(TAlign const& align, std::string& s0, std::string& s1) {
+    int32_t s = -1;
+    int32_t e = -1;
+    uint32_t leadCrop = 0;
+    for(uint32_t j = 0; j<align.shape()[1]; ++j) {
+      if (align[0][j] != '-') {
+	if (align[1][j] != '-') {
+	  if (s == -1) s = j;
+	  e = j + 1;
+	}
+	if (s == -1) ++leadCrop;
+      }
+    }
+    s0.clear();
+    s1.clear();
+    for(int32_t j = s; j < e; ++j) {
+      if (align[0][j] != '-') s0.push_back(align[0][j]);
+      if (align[1][j] != '-') s1.push_back(align[1][j]);
+    }
+    return leadCrop;
+  }
+
 
 }
 
